import pytest


pytest_plugins = "pytester"


@pytest.fixture(autouse=True)
def _divert_atexit(request, monkeypatch):
    import atexit

    atexit_fns = []

    def atexit_register(func, *args, **kwargs):
        atexit_fns.append(lambda: func(*args, **kwargs))

    def finish():
        while atexit_fns:
            atexit_fns.pop()()

    monkeypatch.setattr(atexit, "register", atexit_register)
    request.addfinalizer(finish)



"""Tests for xfail support."""
import os
import signal

import pytest

IS_PYTEST4_PLUS = int(pytest.__version__[0]) >= 4  # noqa: WPS609
FAILED_WORD = "FAILED" if IS_PYTEST4_PLUS else "FAIL"
PYTEST_GTE_7_2 = hasattr(pytest, "version_tuple") and pytest.version_tuple >= (7, 2)  # type: ignore[attr-defined]

pytestmark = pytest.mark.skipif(  # pylint: disable=invalid-name
    not hasattr(os, "fork"),  # noqa: WPS421
    reason="os.fork required",
)


@pytest.mark.parametrize(
    ("is_crashing", "is_strict"),
    (
        pytest.param(True, True, id="strict xfail"),
        pytest.param(False, True, id="strict xpass"),
        pytest.param(True, False, id="non-strict xfail"),
        pytest.param(False, False, id="non-strict xpass"),
    ),
)
def test_xfail(is_crashing, is_strict, testdir):
    """Test xfail/xpass/strict permutations."""
    # pylint: disable=possibly-unused-variable
    sig_num = signal.SIGTERM.numerator

    test_func_body = (
        "os.kill(os.getpid(), signal.SIGTERM)" if is_crashing else "assert True"
    )

    if is_crashing:
        # marked xfailed and crashing, no matter strict or not
        expected_letter = "x"  # XFAILED
        expected_lowercase = "xfailed"
        expected_word = "XFAIL"
    elif is_strict:
        # strict and not failing as expected should cause failure
        expected_letter = "F"  # FAILED
        expected_lowercase = "failed"
        expected_word = FAILED_WORD
    elif not is_strict:
        # non-strict and not failing as expected should cause xpass
        expected_letter = "X"  # XPASS
        expected_lowercase = "xpassed"
        expected_word = "XPASS"

    session_start_title = "*==== test session starts ====*"
    loaded_pytest_plugins = "plugins: forked*"
    collected_tests_num = "collected 1 item"
    expected_progress = f"test_xfail.py {expected_letter!s}*"
    failures_title = "*==== FAILURES ====*"
    failures_test_name = "*____ test_function ____*"
    failures_test_reason = "[XPASS(strict)] The process gets terminated"
    short_test_summary_title = "*==== short test summary info ====*"
    short_test_summary = f"{expected_word!s} test_xfail.py::test_function"
    if expected_lowercase == "xpassed":
        # XPASS wouldn't have the crash message from
        # pytest-forked because the crash doesn't happen
        short_test_summary = " ".join(
            (
                short_test_summary,
                "The process gets terminated",
            )
        )
    reason_string = (
        f"reason: The process gets terminated; "
        f"pytest-forked reason: "
        f"*:*: running the test CRASHED with signal {sig_num:d}"
    )
    if expected_lowercase == "xfailed" and PYTEST_GTE_7_2:
        short_test_summary += " - " + reason_string
    total_summary_line = f"*==== 1 {expected_lowercase!s} in 0.*s* ====*"

    expected_lines = (
        session_start_title,
        loaded_pytest_plugins,
        collected_tests_num,
        expected_progress,
    )
    if expected_word == FAILED_WORD:
        # XPASS(strict)
        expected_lines += (
            failures_title,
            failures_test_name,
            failures_test_reason,
        )
    expected_lines += (
        short_test_summary_title,
        short_test_summary,
    )
    if expected_lowercase == "xpassed" and expected_word == FAILED_WORD:
        # XPASS(strict)
        expected_lines += ("  " + reason_string,)
    expected_lines += (total_summary_line,)

    test_module = testdir.makepyfile(
        f"""
        import os
        import signal

        import pytest

        # The current implementation emits RuntimeWarning.
        pytestmark = pytest.mark.filterwarnings('ignore:pytest-forked xfail')

        @pytest.mark.xfail(
            reason='The process gets terminated',
            strict={is_strict!s},
        )
        @pytest.mark.forked
        def test_function():
            {test_func_body!s}
        """
    )

    pytest_run_result = testdir.runpytest(test_module, "-ra")
    pytest_run_result.stdout.fnmatch_lines(expected_lines)



from setuptools import setup

setup(
    name="pytest-forked",
    use_scm_version=True,
    description="run tests in isolated forked subprocesses",
    long_description=open("README.rst").read(),
    long_description_content_type="text/x-rst",
    license="MIT",
    author="pytest-dev",
    author_email="pytest-dev@python.org",
    url="https://github.com/pytest-dev/pytest-forked",
    platforms=["linux", "osx"],
    packages=["pytest_forked"],
    package_dir={"": "src"},
    entry_points={
        "pytest11": [
            "pytest_forked = pytest_forked",
        ],
    },
    zip_safe=False,
    install_requires=["py", "pytest>=3.10"],
    setup_requires=["setuptools_scm"],
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 7 - Inactive",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: MacOS :: MacOS X",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Utilities",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
    ],
)



import os

import pytest

needsfork = pytest.mark.skipif(not hasattr(os, "fork"), reason="os.fork required")


@needsfork
def test_functional_boxed(testdir):
    p1 = testdir.makepyfile(
        """
        import os
        def test_function():
            os.kill(os.getpid(), 15)
    """
    )
    result = testdir.runpytest(p1, "--forked")
    result.stdout.fnmatch_lines(["*CRASHED*", "*1 failed*"])


@needsfork
def test_functional_boxed_per_test(testdir):
    p1 = testdir.makepyfile(
        """
        import os
        import pytest

        @pytest.mark.forked
        def test_function():
            os.kill(os.getpid(), 15)
    """
    )
    result = testdir.runpytest(p1)
    result.stdout.fnmatch_lines(["*CRASHED*", "*1 failed*"])


@needsfork
@pytest.mark.parametrize(
    "capmode",
    [
        "no",
        pytest.param("sys", marks=pytest.mark.xfail(reason="capture cleanup needed")),
        pytest.param("fd", marks=pytest.mark.xfail(reason="capture cleanup needed")),
    ],
)
def test_functional_boxed_capturing(testdir, capmode):
    p1 = testdir.makepyfile(
        """
        import os
        import sys
        def test_function():
            sys.stdout.write("hello\\n")
            sys.stderr.write("world\\n")
            os.kill(os.getpid(), 15)
    """
    )
    result = testdir.runpytest(p1, "--forked", "--capture=%s" % capmode)
    result.stdout.fnmatch_lines(
        """
        *CRASHED*
        *stdout*
        hello
        *stderr*
        world
        *1 failed*
"""
    )


def test_is_not_boxed_by_default(testdir):
    config = testdir.parseconfig(testdir.tmpdir)
    assert not config.option.forked
